@@ -1,4 +1,8 @@
-from __future__ import print_function, division, absolute_import
+""" TSI{0,1,2,3,5} are private tables used by Microsoft Visual TrueType (VTT)
+tool to store its hinting source data.
+
+TSI5 contains the VTT character groups.
+"""
 from fontTools.misc.py23 import *
 from fontTools.misc.textTools import safeEval
 from . import DefaultTable
@@ -13,8 +17,7 @@ class table_T_S_I__5(DefaultTable.DefaultTable):
 		assert len(data) == 2 * numGlyphs
 		a = array.array("H")
 		a.fromstring(data)
-		if sys.byteorder != "big":
-			a.byteswap()
+		if sys.byteorder != "big": a.byteswap()
 		self.glyphGrouping = {}
 		for i in range(numGlyphs):
 			self.glyphGrouping[ttFont.getGlyphName(i)] = a[i]
@@ -24,8 +27,7 @@ class table_T_S_I__5(DefaultTable.DefaultTable):
 		a = array.array("H")
 		for i in range(len(glyphNames)):
 			a.append(self.glyphGrouping.get(glyphNames[i], 0))
-		if sys.byteorder != "big":
-			a.byteswap()
+		if sys.byteorder != "big": a.byteswap()
 		return a.tostring()
 
 	def toXML(self, writer, ttFont):
